﻿var Promise = require('promise');
var jsMapper = require("./js_diff/index.js");;

function BuildDiffProcess(cfg, diffService, sourceConrolProviders, parsedDependencies) {

    function createBuildDiffPromiseChain(buildArguments) {
        var handlerParams = {
            buildArguments: buildArguments,
            cfg: cfg,
            finalOutput: {
                "meta": {
                    "generated": new Date().valueOf(),
                    "build": buildArguments.build,
                    "commit": buildArguments.commit,
                    "customerId": cfg.customerId,
                    "appName": buildArguments.appname,
                    //"technologies": "nodejs",
                    "technology": "nodejs",
                    "branch": buildArguments.branch,
                    "environment": {}
                },
                files: [],
                dependencies: parsedDependencies || []
            }
        };

        var scmInstance;

        if (buildArguments.workspacepath) {
            var scmType = sourceConrolProviders[buildArguments.scm];
            if (scmType) {
                scmInstance = new scmType(cfg, buildArguments.workspacePath);
                handlerParams.finalOutput.meta.scm = buildArguments.scm;
            }
        }

        if (!buildArguments.commit && scmInstance && !scmInstance.detectCommitVersion) {
            return Promise.reject("'commit' command-line argument is required, since automatic detection by the " + buildArguments.scm + " SCM is not supported");
        }

        var promiseChain = Promise.resolve();

        if (buildArguments.workspacepath) {

            //2. If there is a mapping, get the diffs
            if (scmInstance) {

                //2.1 Get branch history from the SCM
                promiseChain = promiseChain.then(function (mapping) {
                    return loadBranchHistory(handlerParams, scmInstance);
                });

                //2.2 if commit was not specified, automatically detect it                
                if (!buildArguments.commit) {
                    promiseChain = promiseChain.then(function () {
                        return scmInstance.detectCommitVersion(handlerParams).then(function (commit) {
                            if (commit) {
                                handlerParams.finalOutput.meta.commit = commit;
                            }
                        });
                    });
                };
            }

            //3. Call JS mapping handler
            promiseChain = promiseChain.then(function () {
                return jsMapper(handlerParams);
            });
        };

        //4. Submit the final output to the server
        promiseChain = promiseChain.then(function () {
            return diffService.submitMapping(handlerParams.finalOutput);
        });

        promiseChain = promiseChain.then(function () {
            console.log("Build agent finished");
            return Promise.resolve();
        }).catch(function (err) {
            console.log("Build agent error: " + err);
            return Promise.reject(err);
        });

        return promiseChain;
    }

    this.run = function run(buildArguments) {
        return createBuildDiffPromiseChain(buildArguments);
    }

    function loadBranchHistory(handlerParams, scmInstance) {
        return scmInstance.getBranchHistory(handlerParams).then(function (branchLog) {
            if (branchLog) {
                console.log('Branch history loaded (' + branchLog.length + ' commits)');
                handlerParams.finalOutput.meta.history = branchLog;
            }
            return branchLog;
        });
    }
}
module.exports.BuildDiffProcess = BuildDiffProcess;
module.exports.DiffService = require("./new-diff-service.js").NewDiffService;
module.exports.sourceControlProviders = require("./source-control-providers.js");